//
// LDASTools AL - A library collection to provide an abstraction layer
//
// Copyright (C) 2018 California Institute of Technology
//
// LDASTools AL is free software; you may redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 (GPLv2) of the
// License or at your discretion, any later version.
//
// LDASTools AL is distributed in the hope that it will be useful, but
// without any warranty or even the implied warranty of merchantability
// or fitness for a particular purpose. See the GNU General Public
// License (GPLv2) for more details.
//
// Neither the names of the California Institute of Technology (Caltech),
// The Massachusetts Institute of Technology (M.I.T), The Laser
// Interferometer Gravitational-Wave Observatory (LIGO), nor the names
// of its contributors may be used to endorse or promote products derived
// from this software without specific prior written permission.
//
// You should have received a copy of the licensing terms for this
// software included in the file LICENSE located in the top-level
// directory of this package. If you did not, you can view a copy at
// http://dcc.ligo.org/M1500244/LICENSE
//

#ifndef UNEXPECTED_EXCEPTION_HH
#define UNEXPECTED_EXCEPTION_HH

#include <string>
#include <stdexcept>

namespace LDASTools
{
  namespace Error
  {

    /// \brief  A class for capturing information about unexpected exceptions
    ///
    /// This class is intended to be thrown whenever a bad exception
    /// specification is encountered. This will happen if a function
    /// throws an exception that isn't listed in its exception
    /// specification
    ///
    class unexpected_exception : public std::bad_exception {
    public:
      /// \brief  Constructor
      ///
      /// \param what
      ///	The name of the exception that was bad eg. runtime_error
      ///
      /// \param msg
      ///     The what() of the original exception
      unexpected_exception(const std::string& what,
			   const std::string& msg)
	: m_what(what), m_msg(msg) { }

      /// \brief  Destructor
      ///
      ~unexpected_exception() throw() {}

      /// \brief  The name of the bad exception
      ///
      /// Returns the name of the exception thrown by the function
      /// which was not listed in the functions exception specification
      ///
      ///!ret: The name of the bad exception.
      virtual
      const char* what() const throw() { return m_what.c_str(); }

      /// \brief  The what() of the bad exception
      ///
      ///!ret:  The what() of the bad exception
      const char* msg()  const { return m_msg.c_str(); }

    private:
      std::string m_what;
      std::string m_msg;
    };
  } // namespace - Error

} // namespace - LDASTools

#endif // UNEXPECTED_EXCEPTION_HH
